import { spawn, spawnSync, exec } from "child_process";
import path from "path";
import { Logger } from "../utils/logger";
import { cliArgsToObject } from "../utils/cliArgsToObject";
import { writeFileContents } from "../utils/fileUtils";
// import { requestJAR } from './requestJarAPI';
 function runEslintForDiff(cliParams, options = {}) {
  // $(git diff --name-only HEAD HEAD~1 | grep -E '\.(js|jsx)$' | xargs)
  // await requestJAR(cliParams);
  const { diffPath = null, lintEnv = null } = cliArgsToObject(cliParams);
  const format = "json";
  const reportPath = getReportPath();
  const diffJSON = diffPath ? require(getDiffJsonPath(diffPath)) : null;
  const filteredDiffFiles = getDiffFiles(diffJSON);
  var lintFiles = !filteredDiffFiles
    ? [path.resolve(process.cwd(), "uat")]
    : filteredDiffFiles;
  if (lintEnv === "pre-commit") {
    lintFiles = gitDiffFiles();
  }
  return  new Promise((resolve) => {
    const command = [
      "eslint",
      lintFiles.join(" "),
      `--format=${format}`,
      "-o",
      `${reportPath}`,
    ];
    try {
      const lint_process = spawn("npx", command, {
        shell: true,
        cwd: process.cwd(),
      });
      lint_process.stdout.on("data", (data) => {
        Logger.log(Logger.INFO_TYPE, data);
      });
      lint_process.stderr.on("error", (err) => {
        Logger.log(Logger.FAILURE_TYPE, err);
      });
      lint_process.on("error", (errors) => {
        Logger.log(Logger.FAILURE_TYPE, errors);
      });
      lint_process.on("close", async (onclose) => {
        if (diffPath && !(filteredDiffFiles.length === 0)) {
          const filteredJson = await filterReportBasedDiff(
            require(reportPath),
            diffJSON
          );
          writeFileContents(reportPath, JSON.stringify(filteredJson));
        }
        resolve({ status: "success" });
      });
    } catch (error) {
      Logger.log(Logger.INFO_TYPE, error);
      Logger.log(Logger.FAILURE_TYPE, "Execution Failed");
    }
  });
}

function filterReportBasedDiff(resultJson, diffJson) {
  return new Promise((resolve) => {
    if (!diffJson) {
      Logger.log(Logger.FAILURE_TYPE, "git diff json is not given ...");
      resolve(false);
    }
    const impactLineMap = filterDiffJson(diffJson);
    resultJson = resultJson.filter((result) => {
      const isImpactFile = result.filePath;
      if (isImpactFile.includes(".feature")) {
        return false;
      }
      if (!isImpactFile) {
        return false;
      }
      if (impactLineMap.get(isImpactFile)?.length !== 0) {
        result.messages = result.messages.filter((errorMsg) => {
          if (impactLineMap.get(isImpactFile).includes(`${errorMsg.line}`)) {
            return true;
          }
          return false;
        });
        return true;
      }
    });
    resolve(resultJson);
  });
}

function filterDiffJson(diffJson) {
  const diffFilesMap = new Map();
  diffJson?.diffs?.forEach((diff) => {
    const diffMatch = [];
    const regex = /@@(.*?)@@/g;
    let match;
    while ((match = regex.exec(diff.diff)) !== null) {
      diffMatch.push(...splitFilterNumber(match[1]));
    }
    const diffPathKey = path.resolve(getRootLintPath(), diff.new_path);
    if (diffPathKey === undefined) {
      return;
    }
    if (diffPathKey) {
      diffFilesMap.set(diffPathKey, diffMatch);
    }
  });
  return diffFilesMap;
}

function getRootDir() {
  return process.cwd();
}

function getPathFromTestDir(diffPath) {
  if (!diffPath.includes("uat")) {
    return undefined;
  }
  const indexOfDiff = diffPath.split("/").indexOf("uat");
  const diffActualPath = diffPath.split("/").slice(indexOfDiff).join("/");
  return diffActualPath;
}
function splitFilterNumber(numberString) {
  var resultedNum = [];
  numberString
    .split(" ")
    .join(",")
    .split(",")
    .filter((element) => {
      if (element.startsWith("-")) {
        resultedNum.push(element.replace("-", " ").trim());
        // return element.replace('-', ' ').trim();
        return true;
      }

      if (element.startsWith("+")) {
        resultedNum.push(element.replace("+", " ").trim());
        return true;
      }
      return false;
    });
  return resultedNum;
}

function getDiffFiles(diffJson) {
  if (!diffJson) {
    return null;
  }
  const rootPath = getRootLintPath();
  return diffJson?.diffs.map((diff) => {
    return path.resolve(rootPath, diff?.new_path);
  });
}

function getReportPath(reportPath = null, reportOptions = {}) {
  let jsonReportPath =
    reportPath ||
    path.resolve(process.cwd(), "uat", "lint-report", "lintReport.json");
  return jsonReportPath;
}

function getDiffJsonPath(diffJsonPath) {
  return path.resolve(diffJsonPath);
}

function gitDiffFiles() {
  const command = "git diff --cached -U0 HEAD --name-only";
  const child = spawnSync(command, { shell: true });
  const diffFiles = child.stdout
    .toString()
    .split("\n")
    .map((filePath) => path.resolve(getRootLintPath(), filePath));
  diffFiles.pop(); // remove last empty string
  return diffFiles;
}

function getRootLintPath() {
  const childProcess = spawnSync("git", ["rev-parse", "--show-toplevel"], {
    shell: true,
  });
  return childProcess.stdout.toString().split("\n").filter(Boolean).pop();
}

// function validateReport() {
//   if (1) {
//   } else {
//     throw new Error('Report Failed');
//     process.exit(1);
//   }
// }
export {
  runEslintForDiff,
  getDiffFiles,
  getReportPath,
  getDiffJsonPath,
  getRootDir,
  gitDiffFiles,
};
