// Js script to start sonar Server

/**
 *  cli cmd :
 *  npx ZDTestingFramework lint -- --lintEnv=ci --diffPath="" --repo="deskClientApp" --branch="" --username=""
 */

import path from "path";
import { spawn, spawnSync } from "child_process";
import {
  getDiffFiles,
  getDiffJsonPath,
  getReportPath,
  gitDiffFiles,
} from "./eslintLintStage";
import { cliArgsToObject } from "../utils/cliArgsToObject";
import { getSummaryOfLint, writeSummaryOfLint } from "./sonarReportSummary";
import { Logger } from "../utils/logger";
import { getSonarJarPath } from "../utils/rootPath";
import { showMsgPopUp } from "./popUpCli";

function runSonarAnalysis(cliParams) {
  const {
    diffPath,
    repo,
    branch,
    username,
    login = null,
    password = null,
    lintEnv = null,
  } = cliArgsToObject(cliParams);
  const sonarqube_host_url =
    "https://serverlinter-np.zohodesk.csez.zohocorpin.com/";
  const sonar_scanner_path = getSonarJarPath();
  const { Dlogin, Dpassword } = getCredentials(login, password);
  var project_name =
    repo && branch && branch ? `${repo}_${branch}_${username}` : "UAT_Testing";
  if (lintEnv === "pre-commit") {
    project_name = `${getProjectNameFrmGit()}_pre-commit`;
  }
  var resultFilesPath = diffPath
    ? filterSpecAndFeatureFiles(
        getDiffFiles(require(getDiffJsonPath(diffPath)))
      )?.stepFilesPath.join(",")
    : [`${path.resolve(process.cwd(), "uat")}`].join(",");
  var source_directories = diffPath
    ? filterSpecAndFeatureFiles(
        getDiffFiles(require(getDiffJsonPath(diffPath)))
      )?.featureFilesPath
    : [" "];
  if (lintEnv == "pre-commit") {
    resultFilesPath =
      filterSpecAndFeatureFiles(gitDiffFiles()).stepFilesPath.join(",");
  }

  if (lintEnv == "pre-commit") {
    source_directories =
      filterSpecAndFeatureFiles(gitDiffFiles()).featureFilesPath.join(",");
  }

  const command = [
    "-jar",
    sonar_scanner_path,
    `-Dsonar.host.url=${sonarqube_host_url}`,
    `-Dsonar.projectKey=${project_name}`,
    `-Dsonar.login=${Dlogin}`,
    `-Dsonar.password=${Dpassword}`,
    `-Dsonar.sources=${source_directories}`,
    `-Dsonar.tests=${resultFilesPath}`,
    "-Dsonar.language=js",
    `-Dsonar.working.directory=${getSonarArtifactsDir()}`,
    `-Dsonar.eslint.reportPaths=${getReportPath()}`,
    "-Dsonar.javascript.lcov.reportPaths=coverage/lcov-report/lcov.info",
  ];

  const child_process = spawn("java", command);

  child_process.stdout.on("data", (data) => {
    Logger.log(Logger.SUCCESS_TYPE, data);
  });

  child_process.stderr.on("data", (errorOut) => {
    Logger.error(Logger.FAILURE_TYPE, errorOut);
  });

  child_process.on("error", (error) => {
    Logger.error(Logger.FAILURE_TYPE, `Error: ${error.message}`);
  });

  child_process.on("close", (code) => {
    Logger.log(
      Logger.SUCCESS_TYPE,
      `Report to Server  Status - ${code === 0 ? "Success" : "Failure"}`
    );
    writeSummaryOfLint(cliParams);
    Logger.log(
      Logger.INFO_TYPE,
      `report url - ${getSummaryOfLint(cliParams).reportUrl}`
    );
    if (lintEnv == "pre-commit") {
      Logger.log(Logger.SUCCESS_TYPE, "lint triggered from pre-commit ...");
      if (!getLintStatus().status && getLintStatus().message == "failed") {
        Logger.log(Logger.FAILURE_TYPE, "rules violated .. ");
        showMsgPopUp("IssueFound");
        process.exit(1);
      } else {
        showMsgPopUp("NoIssueFound");
      }
    }
  });
}

function filterSpecAndFeatureFiles(resultJson) {
  return resultJson.reduce(
    (filesPath, fileReport) => {
      if (fileReport.includes(".feature")) {
        filesPath.featureFilesPath.push(fileReport);
      } else {
        filesPath.stepFilesPath.push(fileReport);
      }
      return filesPath;
    },
    { stepFilesPath: [], featureFilesPath: [] }
  );
}

function getProjectNameFrmGit() {
  let $repoName = getRepoName();
  let $branchName = getBranchName();
  let $username = getGitUsername();
  return $repoName && $branchName && $username
    ? `${$repoName}_${$branchName}_${$username}`
    : "desk_default_project";
}

function getGitUsername() {
  const userName = getSpawnOutput("git", ["config", "user.email"]);
  return userName.pop().split("@")[0];
}

function getRepoName() {
  const repoName = getSpawnOutput("git", ["remote", "get-url", "origin"]);
  return repoName.pop().split("/").pop().split(".")[0];
}

function getBranchName() {
  var branchName = getSpawnOutput("git", ["rev-parse", "--abbrev-ref", "HEAD"]); // to get current branch
  // branchName = branchName
  //   .filter(($branch) => {
  //     if ($branch.startsWith("*") || $branch.includes("*")) {
  //       return true;
  //     }
  //     return false;
  //   })
  //   .shift();
  return branchName.shift().trim();
}

function getSpawnOutput(command, params = []) {
  const demon_process = spawnSync(command, params, { shell: true });
  return demon_process.stdout.toString().split("\n").filter(Boolean);
}

function getCredentials(login, password) {
  if (login && password) {
    return {
      Dlogin: login,
      Dpassword: password,
    };
  }
  return {
    Dlogin: "developer",
    Dpassword: "developer",
  };
}

function getLintStatus() {
  var reportStatus = true;
  const report = require(getReportPath());
  if (report.length == 0) {
    reportStatus = true;
    return {
      status: true,
      message: "passed",
    };
  }
  if (report.length != 0) {
    for (let i = 0; i < report.length; i++) {
      if (report[i].messages != 0) {
        reportStatus = false;
        break;
      }
    }
    if (reportStatus) {
      return { status: reportStatus, message: "passed" };
    }
    return { status: reportStatus, message: "failed" };
  }
  return { status: reportStatus, message: "passed" };
}

function getSonarArtifactsDir() {
  return path.resolve(process.cwd(), "uat", "lint-report", ".scannerwork");
}

export {
  runSonarAnalysis,
  getProjectNameFrmGit,
  getSpawnOutput,
  getLintStatus,
};

// import scanner from 'sonarqube-scanner';
// import { generateConfigFromFile } from '../core/playwright/readConfigFile';
// const postCallBack = () => process.exit();
// import path from 'path';
// import { getReportPath } from './eslintLintStage';
// const serverHostUrl = 'https://serverlinter-np.zohodesk.csez.zohocorpin.com';
// function sonarQubeIntegrateAnalysis(cliParams, option) {
//   const testDir = path.resolve(process.cwd(), 'uat', 'modules');
//   const eslintResult = getReportPath();
//   const artifactsPath = path.resolve(process.cwd(), 'uat', '.scannerwork');
//   const options = getProperties(
//     cliParams,
//     'developer',
//     'developer',
//     '',
//     testDir,
//     'js',
//     eslintResult,
//     artifactsPath
//   );
//   scanner({ serverUrl: serverHostUrl, options: options }, postCallBack);
// }

// export { sonarQubeIntegrateAnalysis };
